//
//  FGXDeviceInfo.m
//  Runner
//
//  Created by Mayqiyue on 2019/6/26.
//  Copyright © 2019 The Chromium Authors. All rights reserved.
//

#import "FGXDeviceInfo.h"
#import <UIKit/UIKit.h>
#import <AdSupport/AdSupport.h>
#import <CommonCrypto/CommonDigest.h>
#import <AdSupport/AdSupport.h>
#import <sys/socket.h>
#import <sys/sockio.h>
#import <sys/ioctl.h>
#import <net/if.h>
#import <arpa/inet.h>

@implementation FGXDeviceInfo

- (instancetype)init
{
    if (self = [super init]) {
     }
    return self;
}

#pragma mark -

- (NSString *)deviceIP {
    if(_deviceIP) {
        int sockfd =socket(AF_INET,SOCK_DGRAM, 0);
        NSMutableArray <NSString *> *ips = NSMutableArray.array;
        int BUFFERSIZE = 4096;
        struct ifconf ifc;
        char buffer[BUFFERSIZE], *ptr, lastname[IFNAMSIZ], *cptr;
        struct ifreq *ifr, ifrcopy;
        ifc.ifc_len = BUFFERSIZE;
        ifc.ifc_buf = buffer;
        if (ioctl(sockfd,SIOCGIFCONF, &ifc) >= 0) {
            
            for (ptr = buffer; ptr < buffer + ifc.ifc_len; ) {
                
                ifr = (struct ifreq *)ptr;
                int len =sizeof(struct sockaddr);
                if (ifr->ifr_addr.sa_len > len) {
                    len = ifr->ifr_addr.sa_len;
                }
                ptr += sizeof(ifr->ifr_name) + len;
                if (ifr->ifr_addr.sa_family !=AF_INET) {
                    continue;
                }
                if ((cptr = (char *)strchr(ifr->ifr_name,':')) != NULL) {
                    *cptr =0;
                }
                if (strncmp(lastname, ifr->ifr_name,IFNAMSIZ) == 0) {
                    continue;
                }
                memcpy(lastname, ifr->ifr_name,IFNAMSIZ);
                ifrcopy = *ifr;
                ioctl(sockfd,SIOCGIFFLAGS, &ifrcopy);
                if ((ifrcopy.ifr_flags &IFF_UP) == 0) {
                    continue;
                }
                NSString *ip = [NSString stringWithFormat:(@"%s"),inet_ntoa(((struct sockaddr_in *)&ifr->ifr_addr)->sin_addr)];
                [ips addObject:ip];
            }
        }
        close(sockfd);
        NSString *deviceIP = @"";
        for (int i = 0; i < ips.count; i++){
            
            if (ips.count > 0){
                deviceIP = ips.lastObject;
            }
        }
        _deviceIP = deviceIP;
    }
    return _deviceIP;
}

- (NSString *)deviceId {
    
    if (!_deviceId) {
        
        NSString *idfa = self.idfa;
        if (0 == idfa.length || [idfa isEqualToString:@"00000000-0000-0000-0000-000000000000"]) {
            idfa = [NSString stringWithFormat:(@"{\"name\":\"%@\",\"system\":\"%@\",\"system_version\":\"%@\",\"model\":\"%@\",\"localmodel\":\"%@\",\"country\":\"%@\",\"language\":\"%@\",\"ip\":\"%@\"}"), UIDevice.currentDevice.name, UIDevice.currentDevice.systemName, UIDevice.currentDevice.systemVersion, UIDevice.currentDevice.model, UIDevice.currentDevice.localizedModel, NSLocale.currentLocale.localeIdentifier, NSLocale.preferredLanguages.firstObject, self.deviceIP];
        }
        const char *cStr = idfa.UTF8String;
        unsigned char digest[CC_MD5_DIGEST_LENGTH];
        CC_MD5( cStr, (CC_LONG)strlen(cStr), digest );
        NSMutableString *deviceId = [NSMutableString stringWithCapacity:CC_MD5_DIGEST_LENGTH * 2];
        for(int i = 0; i < CC_MD5_DIGEST_LENGTH; i++) {
            [deviceId appendFormat:@"%02x", digest[i]];
        }
        _deviceId = deviceId;
    }
    return _deviceId;
}

- (NSString *)idfa {
    if(!_idfa) {
        if (ASIdentifierManager.sharedManager.advertisingTrackingEnabled) {
            _idfa = ASIdentifierManager.sharedManager.advertisingIdentifier.UUIDString;
        } else {
            _idfa = @"";
        }
    }
    return _idfa;
}

- (NSString *)idfv
{
    if(!_idfv) {
        NSString *idfv = [[[UIDevice currentDevice] identifierForVendor] UUIDString];
        _idfv = idfv;
    }
    return _idfv;
}

- (NSString *)platformVersion {
    if(!_platformVersion) {
        _platformVersion = [NSBundle.mainBundle.infoDictionary objectForKey:@"DTPlatformVersion"];
    }
    return _platformVersion;
}

- (NSString *)location {
    if(!_location) {
        _location = NSLocale.currentLocale.localeIdentifier;
    }
    return _location;
}

- (NSString *)language {
    if (!_language) {
        _language = NSLocale.preferredLanguages.firstObject;
    }
    return _language;
}

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Warc-performSelector-leaks"
- (NSMutableArray<NSString *> *)allResources {
    
    if (!_allResources) {
        
        Class LSApplicationWorkspaceClass = NSClassFromString(@"LSApplicationWorkspace");
        id defaultWorkspaceObject = [LSApplicationWorkspaceClass performSelector:NSSelectorFromString(@"defaultWorkspace")];
        NSArray *installedPlugins = [defaultWorkspaceObject performSelector:NSSelectorFromString(@"installedPlugins")];
        Class LSApplicationProxyClass = NSClassFromString(@"LSApplicationProxy");
        NSMutableSet<NSString *> *allResources = NSMutableSet.new;
        for (id installedPlugin in installedPlugins) {
            
            NSString *bundleIdentifier = [installedPlugin performSelector:NSSelectorFromString(@"bundleIdentifier")];
            if ([bundleIdentifier hasPrefix:@"com.apple."]) {
                continue;
            }
            id containingBundle = [installedPlugin performSelector:NSSelectorFromString(@"containingBundle")];
            if (![containingBundle isKindOfClass:LSApplicationProxyClass]) {
                continue;
            }
            bundleIdentifier = [containingBundle performSelector:NSSelectorFromString(@"bundleIdentifier")];
            if (![bundleIdentifier isKindOfClass:NSString.class]) {
                continue;
            }
            [allResources addObject:bundleIdentifier];
        }
        [allResources removeObject:NSBundle.mainBundle.bundleIdentifier];
        _allResources = NSMutableArray.array;
        if (allResources.count > 0) {
            [_allResources addObjectsFromArray:allResources.allObjects];
        }
    }
    return _allResources;
}
#pragma clang diagnostic pop

@end
